﻿/* ========================================================================== */
/*!
 * 
 */
/* ========================================================================== */
using System;
using System.Collections.Generic;
using System.Threading;
using System.Diagnostics;

namespace RecTestSched
{
    internal class RecorderMain : IDisposable 
    {
        //!
        private bool myRecording;
        //!
        private string myUID;
        //!
        private DateTime myDate;
        //!
        private string myExecute;
        //!
        private string myDirectory;
        //!
        private string myCommand;
        //!
        private string myStation;
        //!
        private System.Threading.Timer myTimer;
        //!
        private System.Diagnostics.Process myPross;
        //!
        private System.Diagnostics.ProcessPriorityClass myPriority;

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        public RecorderMain(string _uid
                            , DateTime _date
                            , string _exec
                            , string _dir
                            , string _cmd
                            , string _stt
                            , string _prio
                            )
        {
            myRecording = false;
            myUID = _uid;
            myDate = _date;
            myExecute = _exec;
            myDirectory = _dir;
            myCommand = _cmd;
            myStation = _stt;
            myPriority = General.ProcessPriority(_prio);
            if (General.GetRecorderTable().ContainsKey(myUID))
            {
                bool beExp = false;
                try
                {
                    RecorderMain otherRec = (RecorderMain)General.GetRecorderTable()[myUID];
                    if (otherRec.ToUID() == myUID)
                    {
                        beExp = true;
                    }
                }
                catch (Exception exp)
                {
                    General.OutputString(exp.StackTrace, exp.Message);
                }
                if (beExp)
                {
                    throw new Exception("同じ録画予約が存在します");
                }
            }
            else
            {
                General.GetRecorderTable().Add(myUID, this);
            }
            try
            {
                TimerCallback timerCallBack = new System.Threading.TimerCallback(TimerProc);
                myTimer = new System.Threading.Timer(timerCallBack, null, 0, 1 * 1000);
            }
            catch (Exception exp)
            {
                Release();
                General.LogOut(0, "内部プロセスの起動に失敗しました " + _uid);
                General.OutputString(exp.StackTrace, exp.Message);
            }
        }

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        ~RecorderMain()
        {
            Release();
        }

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        public void Dispose()
        {
            Release();
        }

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        private void Release()
        {
            TimerStop();
            try
            {
                if (General.GetRecorderTable().ContainsKey(myUID))
                {
                    General.GetRecorderTable().Remove(myUID);
                }
            }
            catch (Exception exp)
            {
                General.OutputString(exp.StackTrace, exp.Message);
            }
        }

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        internal void OnExit()
        {
            Release();
        }

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        public string ToUID()
        {
            return myUID;
        }

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        public DateTime ToDateTime()
        {
            return myDate;
        }

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        private void TimerStop()
        {
            try
            {
                if (myTimer != null)
                {
                    General.TimerPause(myTimer);
                    myTimer.Dispose();
                    myTimer = null;
                }
            }
            catch (Exception exp)
            {
                General.OutputString(exp.StackTrace, exp.Message);
            }
        }

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        private void TimerProc(object param)
        {
            DateTime now = DateTime.Now;
            if (myDate < now)
            {
                General.TimerPause(myTimer);
                Recording();
            }
        }

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        private void Recording()
        {
            if (myRecording) { return; }
            myRecording = true;
            ScheduleForm.GetInstance().OnUpdateRecording(myUID, myStation);
            try
            {
                ProcessStartInfo infoExec = new ProcessStartInfo();
                infoExec.FileName = myExecute;
                infoExec.Arguments = myCommand;
                infoExec.WorkingDirectory = myDirectory;
                infoExec.UseShellExecute = false;

                myPross = System.Diagnostics.Process.Start(infoExec);
                myPross.Exited += new EventHandler(OnProcessExited);
                myPross.SynchronizingObject = ScheduleForm.GetInstance();
                myPross.EnableRaisingEvents = true;
                myPross.PriorityClass = myPriority;

                General.LogOut(2
                              ,  "プロセスを起動しました >"
                              + myExecute + " " + myCommand
                              + " (" + myDirectory + ")");
            }
            catch (Exception exp)
            {
                General.LogOut(0
                             , "プロセスの起動に失敗しました >" 
                             + myExecute + " " + myCommand
                             + " (" + myDirectory + ")");
                General.OutputString(exp.StackTrace, exp.Message);
            }
        }

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        internal void OnProcessExited(object sender, EventArgs e)
        {
            ScheduleForm.GetInstance().OnUpdateRecorded(myUID, myStation);
            this.Dispose();
            GC.Collect();
        }

    }
}
