/* ========================================================================== */
/*!
 * 
 */
/* ========================================================================== */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Windows.Forms;

namespace RecTestSched
{
    partial class ScheduleForm
    {

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        private void ShowDriverWindow()
        {
            tabControlMain.SelectedTab = tabPageDriver;
            this.TopMost = true;
            this.Show();
        }

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        private void DefaultOption()
        {
            checkBoxLog.Checked = false;
            checkBoxTimeAdd.Checked = false;
            checkBoxShiftTime.Checked = false;
            checkBoxTwitter.Checked = false;
            textBoxAddStart.Text = "-10";
            textBoxAddLength.Text = "10";
            textBoxShiftTime.Text = "-5";
            TimeAddUpdate();
            ShiftTimeUpdate();
            TwitterCheckUpdate();
        }

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        private void DefaultComboBox()
        {
            comboBoxPriority.Items.Clear();
            comboBoxPriority.Items.Add("Idle");
            comboBoxPriority.Items.Add("Normal");
            comboBoxPriority.Items.Add("High");
            comboBoxPriority.Items.Add("RealTime");
            comboBoxPriority.SelectedItem = 1;
            comboBoxPriority.Text = "Normal";
            CheckPriorityText();
        }

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        private void DefaultDriverBox()
        {
            buttonDrvUp.Enabled = false;
            buttonDrvDel.Enabled = false;
            textBoxDrvName.Text = "";
            textBoxDrvPath.Text = "";
            textBoxChPath.Text = "";
            textBoxRecCount.Text = "1";
            textBoxAhead.Text = "10";
            textBoxArg.Text = "/d {driver} /chspace {chsp} /rch {ch} /sid {sid} /rec /silent /recexit /recduration {second} /recfile \"c:\\{title}_{did}_{uid}.ts\"";
        }

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        private void ClearDriver()
        {
            DefaultDriverBox();
            dataGridViewDriver.Rows.Clear();
            dataGridViewChannel.Rows.Clear();
            checkBoxRec.Checked = false;
            textBoxRecPath.Text = "";
            comboBoxPriority.SelectedItem = 1;
            comboBoxPriority.Text = "Normal";
            CheckAddTimeText();
            CheckPriorityText();
        }

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        private void DefaultScheduleBox()
        {
            buttonShDelete.Enabled = false;
            comboBoxCh.SelectedIndex = -1;
            comboBoxCh.SelectedText = "";
            textBoxTitle.Text = "";
            comboBoxCh.Text = "";
            textBoxUID.Text = System.IO.Path.GetRandomFileName().Split('.')[0];
            int length = General.ToInt32(textBoxLength.Text);
            if (length < 1)
            {
                textBoxLength.Text = "30";
            }
            dateTimeSchedule.Value = General.PureTime(DateTime.Now.AddMinutes(5));
        }

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        private void ClearSchedule()
        {
            DefaultScheduleBox();
            dataGridViewSchedule.Rows.Clear();
        }

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        private void DriverRowsAdd(string _name
                                  , string _drv
                                  , string _ch
                                  , string _arg
                                  , string _rec
                                  , string _ahe
                                  )
        {
            dataGridViewDriver.Rows.Add(_name, _drv, _ch, _arg, _rec, _ahe);
        }

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        private void DriverRowsDel(int _index)
        {
            try
            {
                if (_index < dataGridViewDriver.Rows.Count)
                {
                    dataGridViewDriver.Rows.RemoveAt(_index);
                    dataGridViewDriver.Refresh();
                }
            }
            catch (Exception exp)
            {
                General.OutputString(exp.StackTrace, exp.Message);
            }
        }

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        private void DriverRowsUpdate(DataGridViewCellCollection _cell
                                   , string _name
                                   , string _drv
                                   , string _ch
                                   , string _arg
                                   , string _rec
                                   , string _ahe
                                   )
        {
            _cell[ColumnName.Index].Value = _name;
            _cell[ColumnDriver.Index].Value = _drv;
            _cell[ColumnChannel.Index].Value = _ch;
            _cell[ColumnArgument.Index].Value = _arg;
            _cell[ColumnProcess.Index].Value = _rec;
            _cell[ColumnAhead.Index].Value = _ahe;
        }

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        private void DriverTextUpdate(int _select)
        {
            DataGridViewCellCollection trgCells = dataGridViewDriver.Rows[_select].Cells;
            textBoxDrvName.Text = General.ToString(trgCells, ColumnName);
            textBoxDrvPath.Text = General.ToString(trgCells, ColumnDriver);
            textBoxChPath.Text = General.ToString(trgCells, ColumnChannel);
            textBoxArg.Text = General.ToString(trgCells, ColumnArgument);
            textBoxRecCount.Text = General.ToString(trgCells, ColumnProcess);
            textBoxAhead.Text = General.ToString(trgCells, ColumnAhead);
            buttonDrvUp.Enabled = true;
            buttonDrvDel.Enabled = true;
        }

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        private void ChannelRowsAdd( string _driver
                                   , string _name
                                   , string _ch
                                   , string _tune
                                   , string _cid
                                   )
        {
             dataGridViewChannel.Rows.Add( _driver, _name, _ch, _tune, _cid );
        }

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        private void ScheduleRowsAdd(string _unique
                                   , string _title
                                   , string _station
                                   , DateTime _date
                                   , int _minute
                                   , string _descript
                                   , string _status
                                   )
        {
            if (_status == null) { _status = ""; }
            dataGridViewSchedule.Rows.Add(_unique, _title, _station, _date, _minute, _descript, _status, 0);
        }

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        private void ScheduleRowsUpdate(DataGridViewCellCollection _cell
                                       , string _unique
                                       , string _title
                                       , string _station
                                       , DateTime _date
                                       , int _minute
                                       , string _descript
                                       , string _status
                                       )
        {
            _cell[ColumnTitle.Index].Value = _title;
            _cell[ColumnDate.Index].Value = _date;
            if (_minute > 0)
            {
                _cell[ColumnRecChannel.Index].Value = _station;
                _cell[ColumnLength.Index].Value = _minute;
                _cell[ColumnDescript.Index].Value = _descript;
                if (_status != null && _status != "")
                {
                    _cell[ColumnStatus.Index].Value = _status;
                }
            }
        }

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        private void ScheduleRowsDel(int _index)
        {
            try
            {
                if (_index < dataGridViewSchedule.Rows.Count)
                {
                    dataGridViewSchedule.Rows.RemoveAt(_index);
                    dataGridViewSchedule.Refresh();
                }
            }
            catch (Exception exp)
            {
                General.OutputString(exp.StackTrace, exp.Message);
            }
        }

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        private void ScheduleRowsSort()
        {
            dataGridViewSchedule.Sort( dataGridViewSchedule.Columns[ColumnDate.Index]
                                     , ListSortDirection.Ascending
                                     );
        }

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        private void ScheduleRowsCrawl()
        {
            DateTime timeBorder = DateTime.Now.AddMinutes(10);
            int index = 0;
            while(true)
            {
                if (index >= dataGridViewSchedule.Rows.Count) { break; }
                DataGridViewRow scRows = dataGridViewSchedule.Rows[index];
                switch (General.ToString(scRows.Cells, ColumnStatus))
                {
                    case "^掸s":
                    case "^抮":
                        DateTime endTime = General.ToDateTime(General.ToString(scRows.Cells, ColumnDate));
                        endTime = endTime.AddMinutes(General.ToInt32(scRows.Cells, ColumnLength));
                        if (endTime < timeBorder)
                        {
                            ScheduleRowsDel(index);
                            --index;
                        }
                        break;
                }
                ++index;
            }
        }

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        private void ScheduleRowsFlash()
        {
            int index = 0;
            while (true)
            {
                if (index >= dataGridViewSchedule.Rows.Count) { break; }
                DataGridViewRow scRows = dataGridViewSchedule.Rows[index];
                switch (General.ToString(scRows.Cells, ColumnStatus))
                {
                    case "^掸s":
                    case "^抮":
                        ScheduleRowsDel(index);
                        --index;
                        break;
                }
                ++index;
            }
        }

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        private void ScheduleRowsClearSel(string _unique)
        {
            int index = 0;
            while (true)
            {
                if (index >= dataGridViewSchedule.Rows.Count) { break; }
                DataGridViewRow scRows = dataGridViewSchedule.Rows[index];
                if (_unique == General.ToString(scRows.Cells, ColumnUID))
                {
                    General.LogOut(2, General.ToString(scRows.Cells, ColumnTitle) + " (" + _unique + ") 폜܂");
                    ScheduleRowsDel(index);
                    --index;
                }
                ++index;
            }
        }

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        private void TimeAddUpdate()
        {
            if (checkBoxTimeAdd.Checked)
            {
                if (!General.IsInt32(textBoxAddStart.Text))
                {
                    textBoxAddStart.Text = "-10";
                }
                textBoxAddStart.ReadOnly = false;

                if (!General.IsInt32(textBoxAddLength.Text))
                {
                    textBoxAddLength.Text = "10";
                }
                textBoxAddLength.ReadOnly = false;
            }
            else
            {
                textBoxAddStart.ReadOnly = true;
                textBoxAddLength.ReadOnly = true;
            }
            CheckAddTimeText();
        }

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        private void ShiftTimeUpdate()
        {
            if (checkBoxShiftTime.Checked)
            {
                if (!General.IsInt32(textBoxShiftTime.Text))
                {
                    textBoxShiftTime.Text = "-5";
                }
                textBoxShiftTime.ReadOnly = false;
            }
            else
            {
                textBoxShiftTime.ReadOnly = true;
            }
            CheckSiftTimeText();
        }
        
        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        private void ScheduleTextUpdate(int _select)
        {
            DataGridViewCellCollection trgCells = dataGridViewSchedule.Rows[_select].Cells;
            textBoxUID.Text = General.ToString(trgCells, ColumnUID);
            textBoxTitle.Text = General.ToString(trgCells, ColumnTitle);
            comboBoxCh.Text = General.ToString(trgCells, ColumnRecChannel);
            dateTimeSchedule.Value = General.ToDateTime(trgCells, ColumnDate);
            int length = General.ToInt32(trgCells, ColumnLength);
            if (length < 1)
            {
                textBoxLength.Text = "30";
            }
            else
            {
                textBoxLength.Text = length.ToString();
            }
            buttonShDelete.Enabled = true;
        }

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        private void TwitterCheckUpdate()
        {
            if (checkBoxTwitter.Checked)
            {
                textBoxTwiName.ReadOnly = false;
                textBoxTwiPass.ReadOnly = false;
            }
            else
            {
                textBoxTwiName.ReadOnly = true;
                textBoxTwiPass.ReadOnly = true;
            }
            TwitterTextUpdate();
        }

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        private void TwitterTextUpdate()
        {
            if (checkBoxTwitter.Checked == false
                ||
                textBoxTwiName.Text == ""
                ||
                textBoxTwiPass.Text == ""
               )
            {
                buttonTwiPost.Enabled = false;
                return;
            }
            buttonTwiPost.Enabled = true;
        }

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        private void CheckAddTimeText()
        {
            if (textBoxAddStart.Text == "+"
                ||
                textBoxAddStart.Text == "-"
              )
            {
            }
            else
            {
                myAddStart = General.ToInt32(textBoxAddStart.Text);
                textBoxAddStart.Text = myAddStart.ToString();
            }
            if (textBoxAddLength.Text == "+"
                ||
                textBoxAddLength.Text == "-"
                )
            {
            }
            else
            {
                myAddLength = General.ToInt32(textBoxAddLength.Text);
                textBoxAddLength.Text = myAddLength.ToString();
            }
            if (checkBoxTimeAdd.Checked == false)
            {
                myAddStart = 0;
                myAddLength = 0;
            }
        }

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        private void CheckSiftTimeText()
        {
            if (textBoxShiftTime.Text == "+"
                ||
                textBoxShiftTime.Text == "-"
              )
            {
            }
            else
            {
                myShiftTime = General.ToInt32(textBoxShiftTime.Text);
                textBoxShiftTime.Text = myShiftTime.ToString();
            }
            if (checkBoxShiftTime.Checked == false)
            {
                myShiftTime = 0;
            }
        }

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        private void CheckPriorityText()
        {
            myPriority = comboBoxPriority.Text;
        }

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        internal bool CheckLogLevel()
        {
            return checkBoxLog.Checked;
        }
    
    }
}
